# rpartパッケージを用いる
require(rpart)

# cartクラスを生成する関数
cart<-function(p,D){ # p: 特徴量の次元，D:木の深さの上限
  model<-list(f=0,maxDepth=D,names=c(paste("X",1:p,sep=""),"Y"))
  class(model)<-'cart'
  return(model)
}

# cartクラスを学習する関数
learn.cart<-function(model,x,y,weight=NULL){
  if(is.null(weight)){
    n<-nrow(x)
    weight<-rep(1/n,n)
  }
  dat<-data.frame(x,y) # rpartに渡すためデータフレームを作る
  colnames(dat)<-model$names
  model$f<-rpart(Y ~ .,data=dat,weights=weight,method="class",
    control=rpart.control(maxdepth=model$maxDepth))
  return(model)
}

# cartクラスで判別を行う関数
classify.cart<-function(model,x){
  p<-ncol(x)
  x<-data.frame(x)
  colnames(x)<-model$names[1:p]
  return(sign(as.numeric(predict(model$f,x,type="class"))-1.5))
}
