# データの判別結果を2次元で表示するための関数
plot.boosted<-function(model,x,y){ 
  z<-classify(model,x) # 判別結果の計算
  plot.discr(x,y,z)
}

# データと判別結果を比較して表示するための関数
plot.discr<-function(x,y,z) { # plot discrimate results
  xrange<-c(floor(min(x[,1])),ceiling(max(x[,1])))
  yrange<-c(floor(min(x[,2])),ceiling(max(x[,2])))
  plot(0,0,xlim=xrange,ylim=yrange,type="n",ann=F)
  title(xlab=expression(x[1]),ylab=expression(x[2]))

  points(x[z<0 & y>0,],pch=d.ch[4],col=d.col[4],bg=d.bg[4],cex=2) # 不正解
  points(x[z>0 & y<0,],pch=d.ch[3],col=d.col[3],bg=d.bg[3],cex=2) # 不正解
  points(x[z<0 & y<0,],pch=d.ch[2],col=d.col[2],bg=d.bg[2]) # 正解
  points(x[z>0 & y>0,],pch=d.ch[1],col=d.col[1],bg=d.bg[1]) # 正解
}

# 3次元表示のための関数plot3を総称関数として宣言
plot3<-function(model,...){
  UseMethod('plot3')
}

# 判別器の俯瞰図を3次元表示するための関数
plot3.boosted<-function(model,x){
  x1<-seq(floor(min(x[,1])),ceiling(max(x[,1])),length=40)
  x2<-seq(floor(min(x[,2])),ceiling(max(x[,2])),length=40)
  z<-outer(x1,x2,function(x,y)classify(model,cbind(x,y)))
  persp(x1,x2,z,theta=30,phi=30,expand=0.5,col=d.persp,
	xlab="x1",ylab="x2",zlab="y",zlim=c(-1,1))
}

# 各モデルでの表示を指定 (この例では全て同一の関数を利用)
plot.stump<-plot.boosted
plot3.stump<-plot3.boosted
plot.cart<-plot.boosted
plot3.cart<-plot3.boosted
plot.ldf<-plot.boosted
plot3.ldf<-plot3.boosted
