# 識別境界を決める関数の定義
boundary<-function( x, y ) {sign(sin(0.75*pi*y)-x)} # Sine関数
# boundary<-function( x, y ) {sign(2-x*x-y*y)} # 円
teacher<-function(x){apply(x,1,function(x){boundary(x[1],x[2])})}

# 例題の作成
rsmpl<-function(n=400) { # [-2,2]x[-2,2]上で一様に生成
  x<-matrix(runif(2*n,min=-2,max=2),n,2)
  y<-teacher(x)
  return(smpl(x,y))
}
dat<-rsmpl() # defaultで400点生成
# dat<-rsmpl(1000) # 1000点生成したい場合

# モデルの宣言
# スタンプを使う場合
# model<-stump() 
# cart (次元2,深さ3) を使う場合
model<-cart(2,3) 
# 線形判別関数を使う場合
# model<-ldf(2)

# AdaBoostの実行
Tmax<-10 
# Tmax<-60 # stumpの場合はこのくらいやった方が良い
boost<-adaboost(model,dat$x,dat$y,T=Tmax,plot=TRUE) # 経過を図示する場合
# boost<-adaboost(model,dat$x,dat$y,T=Tmax) # 途中経過を見ない場合

# 最終結果の表示

dev.new() # 最初の弱学習器での判別結果
plot(boost$learners[[1]],dat$x,dat$y)
title("before AdaBoost")

dev.new() # 最初の弱学習器だけを用いた判別器
plot3(boost$learners[[1]],dat$x)
title(paste("single classifier by",class(model)))

dev.new() # ブースティング後の判別結果
plot(boost,dat$x,dat$y)
title(paste("after AdaBoost (T=",Tmax,")",sep=""))

dev.new() # ブースティングの途中で生成された弱学習器
par(mfrow=c(3,4))
lapply(boost$learners,function(model){plot3(model,dat$x)})

dev.new() # ブースティングの結果得られた判別器
plot3(boost,dat$x)
title("obtained classifier by AdaBoost")
