# stumpクラスを生成する関数
stump<-function(){
  model<-list(index=1,b=0,sgn=1)
  class(model)<-'stump'
  return(model)
}

# stumpクラスを学習する関数
learn.stump<-function(model,x,y,weight=NULL){
  n <- nrow(x) # データ数
  p <- ncol(x) # 次元数
  if(is.null(weight)){ # 重みが指定されていなければ一様な重みにする
    weight <- rep(1/n,n)
  }
  errmin <- 1 # 誤差の最小値を保持する
  for(j in 1:p){ # 各次元について順に調べる
    xx <- sort(x[,j]) # データの並べ替え
    for(i in 1:(n-1)){ # 各データの中間を候補として順に調べる
      ss <- 1
      if(xx[i] != xx[i+1]){ 
        delta <- (xx[i]+xx[i+1])/2
        err <- sum(sign( y != ss*sign(x[,j]-delta))*weight )
        if(err > 0.5){ # 誤りが50%を越えたら判別を反転
          err <- 1-err
          ss <- ss*(-1)
        }
        if(err < errmin){ # 誤差の最小値を達成するものに更新
          model$b <- delta
          model$sgn <- ss
          model$index <- j
          errmin <- err
        }
      }
    } 
  }
  return(model)
}

# stumpクラスで判別を行う関数
classify.stump<-function(model,x){
  return(model$sgn*sign(x[,model$index]-model$b))
}
