# 2次元の教師モデルをランダムに生成
k<-3 # 混合数
teacher<-list()
for (i in 1:k){
  tmp<-gauss(2)
  tmp$mu<-rnorm(2)*2
  tmp$sigma<-crossprod(matrix(rnorm(4),2,2))+diag(0.1,2)
  teacher[[i]]<-tmp
}
teacher<-mixture(teacher)
tmp<-runif(k)+0.3
teacher$prob<-tmp/sum(tmp)

# 教師モデルに従うデータ点を生成
n<-1000 # データ数
x<-rprob(teacher,n)

# 生徒モデルを生成し，ランダムに初期化
h <- k # 混合数 (この例では教師と同数とする)
init<-list()
for (i in 1:h){
  tmp<-gauss(2)
# 制限した正規分布を用いる場合は以下を用いる
#  tmp<-gaussd(2)
#  tmp<-gaussu(2)
# またこれらを混合して使ってもよい
  tmp$mu<-x[sample(n,1),] # データをランダムに選び平均の初期値とする
  init[[i]]<-tmp
}
init<-mixture(init)

# 初期状態の表示
dev.new() # open new window
plot(x,xlab="x",ylab="y",cex=0.5)
plot(teacher,add=TRUE,col="green",lty=2)
plot(init,add=TRUE,col="blue",lty=3)
title("initial states (teacher=dashed,init=dotted)")

# 以降の計算で用いる変数の初期化
l<-c() # 対数尤度の変化を保存する変数
l[1]<-loglike(init,x)
est<-init # 推定値を初期化，初期値はinitに保存

# EMアルゴリズムを実行 (最大100ステップ)
for(i in 1:100){
  est<-emstep(est,x)
  l[i+1]<-loglike(est,x)
  # 途中経過を表示 (注:デバイスによっては全てが終わるまで表示されない場合もある)
  if(i%%5==0){
    plot(x,xlab="x",ylab="y",cex=0.5)
    plot(teacher,add=TRUE,col="green",lty=2)
    plot(init,add=TRUE,col="blue",lty=3)
    plot(est,add=TRUE,col="red",lty=1)
    title("estimation (teacher=dashed,init=dotted,est=solid)")
  }
  #
  if((l[i+1]-l[i])<10^-6){
    break # 対数尤度の上昇が10^-6未満ならfor文を抜け終了
  }
}

dev.new() # 対数尤度の変化を表示
plot(l,type="b",xlab="EM steps",ylab="log-likelihood")
title("evolution of log-likelihood")

# 3Dのための表示領域の計算
xrange=c(floor(min(x[,1])),ceiling(max(x[,1])))
yrange=c(floor(min(x[,2])),ceiling(max(x[,2])))

dev.new() # 教師モデルの俯瞰図
plot3(teacher,xlim=xrange,ylim=yrange);title("teacher model")

dev.new() # 生徒モデルの初期値の俯瞰図
plot3(init,xlim=xrange,ylim=yrange);title("initial model")

dev.new() # 生徒モデルの推定値の俯瞰図
plot3(est,xlim=xrange,ylim=yrange);title("estimated model")

dev.new() # クラス事後確率を用いたデータ点のクラス判別結果の表示
cls <- apply(cprob(est,x),1,which.max)
plot(x,col=1+cls,pch=21+cls,xlab="x",ylab="y")
title("estimated class from conditional probability")
