# 2次元描画 (各モデルを2次元で表示)
#   正規分布の分散をマハラノビス距離が1となる楕円で表示する．
#   関数plot()は既に総称関数として定義されている．
plot.gauss<-function(model,add=F,col="blue",pch=21,lty=1){
  s<-svd(model$sigma)
  q<-s$v%*%diag(sqrt(s$d))%*%t(s$v)
  t<-seq(0,2*pi,length=36) 
  if(add==F) {
    plot(rprob(model,50),type="n",xlab="x",ylab="y")
  }
  lines(t(q%*%t(cbind(cos(t),sin(t)))+model$mu),col=col,lty=lty)
  points(t(model$mu),col=col,pch=pch,bg=col)
}

plot.gaussd<-function(model,...){
  plot(d2gauss(model),...)
}

plot.gaussu<-function(model,...){
  plot(u2gauss(model),...)
}

plot.mixture<-function(model,add=F,col="blue",pch=21,lty=1){
  if(add==F) {
    plot(rprob(model,50),type="n",xlab="x",ylab="y")
  }
  lapply(model$models,
    function(model){plot(model,add=TRUE,col=col,pch=pch,lty=lty)})
}

# 3次元描画 (モデルの確率密度を俯瞰図で表示)
plot3<-function(model,xlim=c(-10,10),ylim=c(-10,10)){
  n<-40
  x<-seq(xlim[1],xlim[2],length=n)
  y<-seq(ylim[1],ylim[2],length=n)
  xx<-rep(x,n)
  yy<-rep(y,each=n)
  z<-matrix(dprob(model,cbind(xx,yy)),n,n)
  persp(x,y,z,theta=30,phi=30,expand=0.5,col="lightblue")
}

# 乱数生成器 (各モデルに従う疑似乱数を生成する)
rprob<-function(model,...){
  UseMethod('rprob')
}

rprob.gauss<-function(model,n){
  d<-length(model$mu)
  s<-svd(model$sigma)
  q<-s$v%*%diag(sqrt(s$d))%*%t(s$v)
  z<-matrix(rnorm(d*n),n,d)
  return(sweep(z%*%q,2,model$mu,"+"))
}

rprob.gaussd<-function(model,n){
  return(rprob(d2gauss(model),n))
}

rprob.gaussu<-function(model,n){
  return(rprob(u2gauss(model),n))
}

rprob.mixture<-function(model,n){
  k<-length(model$models)
  j<-sample(1:k,n,replace=TRUE,prob=model$prob)
  z<-NULL
  for(i in 1:k){
    m<-sum(j==i)
    if(m>0){
      z<-rbind(z,rprob(model$models[[i]],m))
    }
  }
  return(z)
}
