decode <- function(res.bin,y,coder,model=1,eps=10^(-2)){
  ## 関数learn.binary()で学習した結果res.binを用いて
  ## 確率モデルに基づく復号器を学習する関数
  ## 入力 (res.bin: learn.binaryの結果, y: 教師ラベル, coder: 符号表,
  ##      model: モデル１あるいはモデル２の切り替え, eps: 正則化定数)
  ## 出力 (B[行列]: 推定したパラメーター, class: 予測クラスラベル, error: 誤り率,
  ##      model: 用いた確率モデル, P.0: クラスラベルの事前確率, P: 事後確率)
  class <- ncol(coder)
  z.tilde <- sign(res.bin$F) # 実数値から+1,-1へ変換
  n <- nrow(z.tilde)
  p <- ncol(z.tilde)
  y0 <- match(y,res.bin$INFO$level.y)

  if(model==1){ # モデル１を用いる場合のパラメーター推定
    B <- matrix(0,p,3) 
    Z <- c(1,0,-1)
    for(i in 1:3){
      R0 <- t(coder[,y0])==Z[i]
      R.n <- apply(R0,2,sum)
      R <- apply(R0*(z.tilde==1),2,sum)/R.n
      B[,i] <- 0.5*log((R+eps/R.n)/(1-R+eps/R.n))
    }
  }
  if(model==2){ # モデル２を用いる場合のパラメーター推定
    R <- apply(as.matrix(c(1:class)),1,
                 function(z){apply(z.tilde[y0==z,]==1,2,mean)})
    R.n <- 1/matrix(table(y0),p,class,byrow=TRUE)
    B <- 0.5*log((R+eps*R.n)/(1-R+eps*R.n))
  }
  
  P.0 <- res.bin$P.0 # 事前確率
  log.P <- matrix(0,n,class) 
  for(g in 1:class){ # 事後確率の対数を計算
    if(model==1){ # モデル1の場合
      B0 <- B[cbind(1:p,match(coder[,g],Z))]
      log.P[,g] <- z.tilde %*% B0 -sum(log(exp(B0)+exp(-B0)))+log(P.0[g])
    }
    if(model==2){ # モデル2の場合
      log.P[,g] <- z.tilde %*% B[,g]
                       -sum(log(exp(B[,g])+exp(-B[,g])) )+log(P.0[g])
    }
  }
  class <- res.bin$INFO$level.y[max.col(log.P)] # クラス予測
  
  res <- list(B=B,class=class,error=mean(class!=y),model=model,
              P.0=P.0,P=exp(log.P)/apply(exp(log.P),1,sum))
  class(res) <- 'decode'
  return(res)

}
