decode2 <- function(res.bin,y,coder,T=5000,S=100000,lam=0){
  ## 関数learn.binary()で学習した結果res.binを用いて
  ## 確率モデル3に基づく復号器を学習する関数
  ## 入力 (res.bin: learn.binaryの結果, y: 教師ラベル, coder: 符号表,
  ##      T: 勾配法の更新回数, S: 正則化定数の計算に用いるサンプル数, lam: 正則化定数)
  ## 出力 (P.0: クラスラベルの事前確率, par:[リスト]: 推定したパラメーター,
  ##      Z: 正規化定数, Z.arg.max: 正規化定数の各要素の最大値,
  ##      P: 事後確率, P.m: 正規化定数の計算に平均場近似を用いた場合の事後確率,
  ##      logZ.m: 平均場近似を用いた場合の正規化定数の対数,
  ##      C: 平均場近似を用いた場合のパラメーター,
  ##      class: クラスラベル予測, class.m: 平均場近似によって復号した場合の予測ラベル,
  ##      error: classの誤り率, error.m: class.mの誤り率)
  
  class <- ncol(coder)
  z.tilde <- sign(res.bin$F)
  n <- nrow(z.tilde)
  p <- ncol(z.tilde)
  y0 <- match(y,res.bin$INFO$level.y)
  
  P.0 <- res.bin$P.0
  P <- matrix(0,n,class)
  P.m <- matrix(0,n,class)
  
  PAR <- list()
  Z <- list()
  logZ.m <- list()
  Z.arg.max <- rep(0,class)
  wvar <- list()
  C <- list() 
  
  for(g in 1:class){
    #　コントラスティブ　ダイバージェンスを用いた勾配法によるパラメータ学習
    PAR[[g]] <- learn.cont.div(z.tilde[y0==g,],T,lam=0)
    A <- PAR[[g]]$A
    B <- PAR[[g]]$B

    # 平均場近似による事後確率の計算
    Q.m <- apply(z.tilde[y0==g,],2,mean)+runif(p,min=-0.15,max=0.15)
    for(i in 1:100){
      Q <- exp(A+B %*% Q.m)
      Q.m <- (Q^2-1)/(Q^2+1)
      Q.m[is.finite(Q.m)==FALSE] <- 1
    }
    wvar[[g]] <- as.matrix(Q.m)
    C[[g]] <- B %*% wvar[[g]] +A  
    logZ.m[[g]] <- sum(log(exp(C[[g]])+exp(-C[[g]])))
    P.m[,g] <- P.0[g]*exp(z.tilde %*% as.matrix(C[[g]])-logZ.m[[g]])

    # MCMCサンプリングによる正規化定数の計算
    R <- 1/(1+exp(-2*C[[g]]))
    Z.cand <- apply(R,1,function(z){
      sample(c(1,-1),S,prob=c(z,1-z),replace=TRUE)})
    Z.arg <- calc.exp.arg(Z.cand,A,B)-Z.cand %*% C[[g]] +logZ.m[[g]]
    Z.arg.max[g] <- max(Z.arg)
    Z[[g]] <- sum(exp(Z.arg-Z.arg.max[g]))
    P[,g] <- P.0[g]*exp(calc.exp.arg(z.tilde,A,B)-Z.arg.max[g])/Z[[g]]  
  }

  P <- P/apply(P,1,sum) # 事後確率の正規化
  P.m <- P.m/apply(P.m,1,sum) # 事後確率の正規化
  class <- res.bin$INFO$level.y[max.col(P)] # クラス予測
  class.m <- res.bin$INFO$level.y[max.col(P.m)] # クラス予測
  res <- list(P.0=P.0,par=PAR,Z=Z,Z.arg.max=Z.arg.max,P=P,
              P.m=P.m,logZ.m=logZ.m,C=C,
              class=class,class.m=class.m,
              error=mean(class !=y),error.m=mean(class.m !=y))
  class(res) <- 'decode2'
  return(res)
}
