source("b2m-make.info.r") # データの情報作成をする関数の読み込み
source("b2m-trans.data.r") # データを変換する関数の読み込み
source("b2m-generate.code.matrix.r") # 符号表を作成する関数の読み込み
source("b2m-learn.binary.r") # 2値判別器の学習を行う関数の読み込み
source("b2m-predict.binary.r") # 学習した2値判別器で予測を行う関数の読み込み
source("b2m-decode.r") # 確率モデルに基づく復号法を行う関数の読み込み
source("b2m-predict.decode.r") # 確率モデルに基づく復号法で予測を行う関数の読み込み
library(mlbench) # データセットの用意
data(Glass) # Glassデータの読み込み
P <- 10
set.seed(1)
index <- sample(nrow(Glass),nrow(Glass)*0.7) # 訓練データ集合の選別
coder <- generate.code.matrix(length(levels(Glass[,P])),"1v1") # 符号表の生成
# ２値判別器の訓練，ガウシアンカーネルを用いたサポートベクターマシン
bin.tra.r <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="rbfdot")
# テストデータに対する２値判別器の予測
bin.gen.r <- predict(bin.tra.r,Glass[-index,-P])

# model 1を用いた復号, 訓練データの予測などを含む
res.tra <-decode(bin.tra.r,Glass[index,P],coder,model=1)
# 学習した復号器を用いて，テストデータの結果を予測
res.gen <- predict(res.tra,bin.gen.r,coder,Glass[-index,P])
res.tra$error # model 1の訓練誤差
res.gen$error # model 1の予測誤差

# model 2を用いた復号, 訓練データの予測などを含む
res2.tra <-decode(bin.tra.r,Glass[index,P],coder,model=2)
# 学習した復号器を用いて，テストデータの結果を予測
res2.gen <- predict(res2.tra,bin.gen.r,coder,Glass[-index,P])
res2.tra$error # model 2の訓練誤差
res2.gen$error # model 2の予測誤差

# ２値判別器の訓練，1次の多項式カーネルを用いたサポートベクターマシン
bin.tra.p <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="polydot")
# テストデータに対する２値判別器の予測
bin.gen.p <- predict(bin.tra.p,Glass[-index,-P])

# model 1を用いた復号, 訓練データの予測などを含む
res.p.tra <-decode(bin.tra.p,Glass[index,P],coder,model=1)
# 学習した復号器を用いて，テストデータの結果を予測
res.p.gen <- predict(res.p.tra,bin.gen.p,coder,Glass[-index,P])
res.p.tra$error # model 1の訓練誤差
res.p.gen$error # model 1の予測誤差

# model 2を用いた復号, 訓練データの予測などを含む
res2.p.tra <-decode(bin.tra.p,Glass[index,P],coder,model=2)
# 学習した復号器を用いて，テストデータの結果を予測
res2.p.gen <- predict(res2.p.tra,bin.gen.p,coder,Glass[-index,P])
res2.p.tra$error # model 2の訓練誤差
res2.p.gen$error # model 2の予測誤差
