source("b2m-make.info.r") # データの情報作成をする関数の読み込み
source("b2m-trans.data.r") # データを変換する関数の読み込み
source("b2m-generate.code.matrix.r") # 符号表を作成する関数の読み込み
source("b2m-learn.binary.r") # 2値判別器の学習を行う関数の読み込み
source("b2m-predict.binary.r") # 学習した2値判別器で予測を行う関数の読み込み
source("b2m-decode2.r") # 確率モデルに基づく復号法を行う関数の読み込み
source("b2m-predict.decode2.r") # 確率モデルに基づく復号法で予測を行う関数の読み込み
source("b2m-calc.exp.arg.r")
source("b2m-learn.cont.div.r") # コントラスティブダイバージェンスを学習する関数

library(mlbench) # データセットの用意
data(Glass)
P <- 10
set.seed(1)
# 訓練データ集合
index <- sample(nrow(Glass),nrow(Glass)*0.7)
# 符号表の生成
coder <- generate.code.matrix(length(levels(Glass[,P])),"1v1")
# ２値判別器の訓練，ガウシアンカーネルを用いたSVM
bin.tra.r <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="rbfdot")
# テストデータに対する２値判別器の予測
bin.gen.r <- predict(bin.tra.r,Glass[-index,-P])

res.tra.r <-decode2(bin.tra.r,Glass[index,P],coder) # 復号器の学習
# テストデータに対する予測
res.gen.r <- predict(res.tra.r,bin.gen.r,coder,Glass[-index,P]) 
res.tra.r$error # 訓練誤差
res.tra.r$error.m # 平均場近似を用いて復号した時の訓練誤差
res.gen.r$error # 予測誤差
res.gen.r$error.m # 平均場近似を用いて復号した時の予測誤差

# ２値判別器の訓練，1次の多項式カーネルを用いたSVM
bin.tra.p <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="polydot")
# テストデータに対する２値判別器の予測
bin.gen.p <- predict(bin.tra.p,Glass[-index,-P]) 
res.tra.p <-decode2(bin.tra.p,Glass[index,P],coder) # 復号器の学習
# テストデータに対する予測
res.gen.p <- predict(res.tra.p,bin.gen.p,coder,Glass[-index,P])
res.tra.p$error # 訓練誤差
res.tra.p$error.m # 平均場近似を用いて復号した時の訓練誤差
res.gen.p$error # 予測誤差
res.gen.p$error.m # 平均場近似を用いて復号した時の予測誤差

