source("b2m-make.info.r") # データの情報作成をする関数の読み込み
source("b2m-trans.data.r") # データを変換する関数の読み込み
source("b2m-generate.code.matrix.r") # 符号表を作成する関数の読み込み
source("b2m-learn.binary.r") # 2値判別器の学習を行う関数の読み込み
source("b2m-predict.binary.r") # 学習した2値判別器で予測を行う関数の読み込み
source("b2m-predict.hamming.r") # ハミング復号法を行う関数の読み込み
library(mlbench) # データセットの準備
data(Glass)
P <- 10
set.seed(1)
# 訓練データセットのインデックス
index <- sample(nrow(Glass),nrow(Glass)*0.7)
# 1 vs 1の符号表の生成
coder <- generate.code.matrix(length(levels(Glass[,P])),"1v1") 
# ２値判別器の訓練，ガウシアンカーネルを用いたサポートベクターマシンを学習
bin.tra.r <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="rbfdot")
sapply(bin.tra.r$Fx,function(z){z@error}) # 学習した各２値判別器の訓練誤差
predict.hamming(bin.tra.r,coder,Glass[index,P])$error # ハミング復号法の訓練誤差

# predict.binary()を使って学習した２値判別関数を用いてテストデータのz-tildeを計算
bin.gen.r <- predict(bin.tra.r,Glass[-index,-P]) 
# ハミング復号の予測誤差
predict.hamming(bin.gen.r,coder,Glass[-index,P])$error

# ２値判別器の訓練，1次の多項式カーネルを用いたサポートベクターマシン
bin.tra.p <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="polydot")
sapply(bin.tra.p$Fx,function(z){z@error}) # ２値判別器の誤り率

predict.hamming(bin.tra.p,coder,Glass[index,P])$error # ハミング復号の訓練誤差

# predict.binary()を使って学習した２値判別関数を用いてテストデータのz-tildeを計算
bin.gen.p <- predict(bin.tra.p,Glass[-index,-P])
predict.hamming(bin.gen.p,coder,Glass[-index,P])$error # ハミング復号の予測誤差

