source("b2m-make.info.r") # データの情報作成をする関数の読み込み
source("b2m-trans.data.r") # データを変換する関数の読み込み
source("b2m-generate.code.matrix.r") # 符号表を作成する関数の読み込み
source("b2m-learn.binary.r") # 2値判別器の学習を行う関数の読み込み
source("b2m-predict.binary.r") # 学習した2値判別器で予測を行う関数の読み込み
source("b2m-predict.loss.decoding.r") # ロス関数による復号法を行う関数の読み込み

library(mlbench) # mlbenchパッケージの読み込み
data(Glass) # glassデータの読み込み
P <- 10
set.seed(1)
#　訓練データセットのインデックスを選択
index <- sample(nrow(Glass),nrow(Glass)*0.7)
# 1 vs 1の符号表を生成
coder <- generate.code.matrix(length(levels(Glass[,P])),"1v1") 
# ガウシアンカーネルを用いて2値判別器を学習
bin.tra.r <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="rbfdot")
# テストデータに対する2値判別器の予測
bin.gen.r <- predict(bin.tra.r,Glass[-index,-P])

# 指数ロス
predict.loss.decoding(bin.tra.r,coder,1,Glass[index,P])$error # 訓練誤差
predict.loss.decoding(bin.gen.r,coder,1,Glass[-index,P])$error # 予測誤差

# ロジスティックロス
predict.loss.decoding(bin.tra.r,coder,2,Glass[index,P])$error # 訓練誤差
predict.loss.decoding(bin.gen.r,coder,2,Glass[-index,P])$error # 予測誤差

# 1次の多項式カーネルを用いて2値判別器を学習
bin.tra.p <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="polydot")
# テストデータに対する2値判別器の予測
bin.gen.p <- predict(bin.tra.p,Glass[-index,-P])

# 指数ロス
predict.loss.decoding(bin.tra.p,coder,1,Glass[index,P])$error # 訓練誤差
predict.loss.decoding(bin.gen.p,coder,1,Glass[-index,P])$error # 予測誤差
# ロジスティックロス
predict.loss.decoding(bin.tra.p,coder,2,Glass[index,P])$error # 訓練誤差
predict.loss.decoding(bin.gen.r,coder,2,Glass[-index,P])$error # 予測誤差
