source("b2m-make.info.r") # データの情報作成をする関数の読み込み
source("b2m-trans.data.r") # データを変換する関数の読み込み
source("b2m-generate.code.matrix.r") # 符号表を作成する関数の読み込み
source("b2m-learn.binary.r") # 2値判別器の学習を行う関数の読み込み
source("b2m-predict.binary.r") # 学習した2値判別器で予測を行う関数の読み込み
source("b2m-learn.prob.r") # 確率値を計算する関数
source("b2m-predict.prob.r")# 確率値を予測する関数
source("b2m-pairwise.r") # ペアワイズカップリングによる復号法

library(mlbench) # データセットの用意
data(Glass)
P <- 10
set.seed(1)
# 訓練データ集合
index <- sample(nrow(Glass),nrow(Glass)*0.7)
# 符号表の生成
coder <- generate.code.matrix(length(levels(Glass[,P])),"1v1")
# ２値判別器の訓練，ガウシアンカーネルを用いたサポートベクターマシン
bin.tra.r <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="rbfdot")
# テストデータに対する２値判別器の予測
bin.gen.r <- predict(bin.tra.r,Glass[-index,-P])
# 確率値の計算
prob.tra.r <- learn.prob(bin.tra.r,Glass[index,P],coder)
# テストデータに対する確率値の計算
prob.gen.r <- predict(prob.tra.r,bin.gen.r)

# 訓練データのペアワイズカップリングによる予測
res.tra.r <- pairwise(prob.tra.r,coder,Glass[index,P])
# テストデータのペアワイズカップリングによる予測
res.gen.r <- pairwise(prob.gen.r,coder,Glass[-index,P])
res.tra.r$error # 訓練誤差
res.gen.r$error # 予測誤差

# ２値判別器の訓練，1次の多項式カーネルを用いたサポートベクターマシン
bin.tra.p <- learn.binary(Glass[index,-P],Glass[index,P],coder,
                          svm.ker="polydot")
# テストデータに対する２値判別器の予測
bin.gen.p <- predict(bin.tra.p,Glass[-index,-P])
# 確率値の計算
prob.tra.p <- learn.prob(bin.tra.p,Glass[index,P],coder)
# テストデータに対する確率値の計算
prob.gen.p <- predict(prob.tra.p,bin.gen.p)

# 訓練データのペアワイズカップリングによる予測
res.tra.p <- pairwise(prob.tra.p,coder,Glass[index,P])
# テストデータのペアワイズカップリングによる予測
res.gen.p <- pairwise(prob.gen.p,coder,Glass[-index,P])
res.tra.p$error # 訓練誤差
res.gen.p$error # 予測誤差
