generate.code.matrix <- function(G,mode="full",
                                 P=ceiling(15*log2(G)),sp.itr.num=10000){
  ## 符号表を作成する関数
  ## 入力 (G: クラス数, mode: 符号表の種類(1R,1v1,sparse),
  ##      P: mode="sparse"の時，何個の2値判別問題を生成するか,
  ##      sp.tir.num: sparseを選択した場合の探索数)
  ## 出力 (coder[行列]: 符号表)
    
  transform.number <- function(p,class=2,G=floor(log(p,base=class)+1)){
    ## 実数pを長さGのclass(デフォルトは2)進数に変換するローカル関数
    ## 入力 (p: 正の整数, class: class進数への変換, G: 長さG)
    ## 出力 : pをclass進数へ変換したベクトル
    if(p> (class)^G-1){
      print("G is too small")
    }
    else{
      base <- class^(c(G:1)-1)
      return((p %/% base) %% class)
    }
  }

  if(mode=="1R"){ # 1 vs R符号
    coder <- matrix(1,G,G)
    diag(coder) <- -1
  }
  if(mode=="1v1"){ #   ## 1 vs 1　符号
    coder <- matrix(0,G*(G-1)/2,G)
    k <- 1
    for(i in 1:(G-1)){
      for(j in (i+1):G){
        coder[k,i] <- 1
        coder[k,j] <- -1
        k <- k+1
      }
    }
  }
  if(mode=="sparse"){ # スパースランダム符号
    random.code <- function(P){
      ## ランダムにP * Gのスパース（1,-1の確率が1/4, 0の確率が1/2）な符号表を
      ## 生成するローカル関数
      ## 入力 (P: ２値判別問題の数)
      ## 出力 (符号表)
      t(apply(as.matrix(c(1:P)),1,
              function(z){sample(c(1,-1,0),G,replace=TRUE,prob=c(1/4,1/4,1/2))}))
    }
    omit.dup <- function(C){
      ## 符号表Cを受け取り無意味なものと重複したものを取り除くローカル関数
      ## 入力 (C: 符号表)
      ## 出力 (C: 重複，意味の無いものを除いた符号表)
      cond <- abs(C %*% 10^c((G-1):0))
      C <- C[match(unique(cond),cond),,drop=FALSE]
      condition0 <- apply(C!=0,1,sum)
      condition1 <- apply(C==1,1,sum)
      condition2 <- apply(C==-1,1,sum)
      condition3 <- apply(C==0,1,sum)
      C <- C[condition1!=condition0 &
             condition2 != condition0 &
             condition3!=G ,,drop=FALSE]
      return(C)
    }
    
    if(P>=(3^G-2^(G+1)+1)/2){ #所望の長さが最大値を超える場合は"full"を使う
      coder <- generate.code.matrix(G,"full")
    }
    else{
      D.max <- 0    
      for(i in 1:sp.itr.num){
        C <- random.code(P)
        C <- omit.dup(C)
        # Cの行数が所望の長さになるまで繰り替えす        
        while(nrow(C)< P | length(C)==0){
          C <- rbind(C,random.code(P))
          C <- omit.dup(C)
        }
        C <- C[1:P,]
        D <- apply(as.matrix(1:G),1,function(z){apply(C[,z]!=C,2,sum)})
        diag(D) <- nrow(C)
        if(min(D)>D.max){
          C.max <- C
          D.max <- min(D)
        }
      }
      coder <- C.max
    }
  }
  if(mode=="full"){ # 全ての候補を含む符号表
    coder <- NULL
    for(i in 0:(3^G-1)){
      res.trans <- as.vector(transform.number(i,3,G)-1)
      if(sum(res.trans==1)*sum(res.trans==-1)==0){
      }
      else{
        coder <- rbind(coder,res.trans)
      }
    }
    coder <- coder[1:(nrow(coder)/2),]
    ind1 <- apply(coder==0,1,sum)
    ind2 <- abs(apply(coder,1,sum))
    coder <- rbind(coder[ind1==0 & ind2==(G-2),],coder[ind1==0 & ind2!=(G-2),],
                   coder[ind1>0,])
  }
  
  rownames(coder) <- paste("binary problem",c(1:nrow(coder)))
  colnames(coder) <- paste("class",1:G)
  return(coder)
}
