learn.binary <- function(x,y,coder,svm.ker="rbfdot",svm.C=1,na.action="all"){
  ## 入力x, ラベルyと符号表coderを受け取り，対応する2値判別器を学習する関数
  ## 入力 (x: 入力, y: 教師ラベル, coder: 符号表,
  ##      svm.ker: カーネル関数(kernlab参照）, svm.C: 正則化パラメーター,
  ##      na.action: 欠測があった場合の対応("all" or "remove"))
  ## 出力 (INFO: 入力，ラベルの情報, Fx[リスト]: 学習した２値判別器,
  ##      F[行列]: 入力xに対するFxの出力, method: 2値判別器として用いた方法,
  ##      P.0: クラスラベルの事前確率)

  INFO <- make.info(x,y) # 入力，ラベル情報の作成
  dat <- trans.data(x,y,INFO,na.action) # データの加工
  INFO$na.index <- dat$na.index
  INFO$na.action <- na.action

  n <- nrow(dat$x)
  CLASS <- ncol(coder)  
  F <- matrix(0,nrow(x),nrow(coder)) 
  Fx <- list()             
  library(kernlab)
  
  # 符号表に従ってサポートベクターマシンを学習
  for(i in 1:nrow(coder)){
    # 符号が0ではない訓練データのインデックス
    tra.ind <- which(apply(as.matrix(dat$y),1,
                           function(z){any(z==which(coder[i,]!=0))}))
    # サポートベクターマシンの学習
    Fx[[i]] <- ksvm(dat$x[tra.ind,,drop=FALSE],coder[i,dat$y[tra.ind]],
                    type="C-svc",kernel=svm.ker,C=svm.C)
    # 判別関数の値を計算
    F[,i] <- predict(Fx[[i]],trans.data(x,y,INFO,na.action="all")$x,
                     type="decision")
  }

  P.0 <- table(dat$y)/n # クラスの事前確率の推定値
  res <- list(INFO=INFO,Fx=Fx,F=F,method="svm",P.0=P.0)
  class(res) <- 'binary'
  return(res)
}
