learn.cont.div <- function(z,T,lam=0){
  ## 入力x, ラベルyと符号表coderを受け取り，対応する2値判別器を学習する関数
  ## 入力 (z[行列]: {+1,-1}からなるp次元の横ベクトルをn個並べた行列,
  ##      T: 更新回数, lam: 正則化パラメーター)
  ## 出力 (A: 学習したパラメーターベクトル，B: 学習したパラメーター行列)

  gibs <- function(A,B,z,N=1,N0=0){
    ## パラメータA,Bで決まるボルツマン分布からギブスサンプリングを行うローカル関数
    ## 入力(A: ボルツマン分布のパラメータ, B: ボルツマン分布の2次のパラメータ,
    ##     z: 初期値,N: サンプリングするステップ数,N0: 始めのN0ステップを捨てる)
    p <- length(A)
    n <- nrow(z)
    Z <- array(0,c(n,p,N))
    for(j in 1:(N+N0)){
      for(i in sample(p,p)){
        theta <- A[i]+z %*% as.matrix(B[i,])
        PROB <- 1/(1+exp(-2*theta))
        z[,i] <- apply(as.matrix(PROB),1,
                       function(z){sample(c(1,-1),1,prob=c(z,1-z))})
      }
      if(j>N0){
        Z[,,j-N0] <- z
      }
    }
    return(Z)
  }
  
  p <- ncol(z)
  n <- nrow(z)
  A <- rep(0,p)
  B <- matrix(0,p,p)
  eta <- 0.01 # 学習係数
  for(t in 1:T){
    z.new <- gibs(A,B,z,1,0)[,,1] # ギブスサンプリング
    gr1 <- apply(z-z.new,2,mean)-lam*sign(A)/(n) # 勾配計算
    gr2 <-  (t(z) %*% z-t(z.new) %*% z.new)/n-lam*sign(B)/(n) # 勾配計算
    A <- A+eta*gr1 #パラメータ更新
    B <- B+eta*gr2 #パラメータ更新
    B <- (B+t(B))/2 # 対称化
    diag(B) <- 0
  } 
  return(list(A=A,B=B))
}
