learn.prob <- function(res.binary,y,coder){
  ## 学習した２値判別器learn.binary，ラベルy，符号表coderを受け取り，
  ## 対応する２値ラベルの事後確率値を学習する関数
  ## 入力 (res.binary: learn.binaryの結果, y: 教師ラベル, coder: 符号表)
  ## 出力 (p.model: 学習したロジスティックモデル，PROB[行列]: ２値の事後確率,
  ##      INFO: 入力，出力の情報, n: 各２値判別問題で訓練に用いた例題数)

  F <- res.binary$F # 2値判別関数の出力値
  INFO <- res.binary$INFO # データの情報
  dat.y <- match(y,INFO$level.y) # データの加工

  PROB <- matrix(0,nrow(F),nrow(coder))
  p.model <- list()
  PRIOR <- list()
  n <- length(nrow(coder))
  
  for(i in 1:nrow(coder)){ # ロジスティックモデルを用いて学習
    tra.ind <- which(apply(as.matrix(dat.y),1,
                           function(z){any(z==which(coder[i,]!=0))}))
    p.model[[i]] <- glm(y~.,
                 data.frame(F=F[tra.ind,i],y=(coder[i,dat.y[tra.ind]]+1)/2),
                        family=binomial)
    n[i] <- length(tra.ind)
    PROB[,i] <- predict(p.model[[i]],data.frame(F=F[,i]),type="response")
  }
  
  res <- list(p.model=p.model,PROB=PROB,INFO=res.binary$INFO,n=n)
  class(res) <- 'prob'
  return(res)
}
