pairwise <- function(res.prob,coder,y=NULL,T=100,eps=10^(-10)){
  ## ペアワイズカップリング法による復号を行う関数
  ## 入力 (res.bin: learn.binary() あるいは predict.binary()の結果, coder: 符号表,
  ##      y: 多値のクラスラベル, T:　更新ステップ数, eps: 許容する推定誤差)
  ## 出力 (class: 予測されたクラスラベル, error: 誤り率)
  G <- ncol(coder)
  N <- nrow(res.prob$PROB)
  yhat <- rep(0,N)
  n <- res.prob$n

  for(j in 1:N){
    KL <- function(p){ # カルバックライブラー擬距離
      p <- p^2/sum(p^2)
      P <- cbind(apply(t(coder==1) *p,2,sum),apply(t(coder==-1) *p,2,sum))
      P <- P/apply(P,1,sum)
      R <- cbind(res.prob$PROB[j,],1-res.prob$PROB[j,])
      D <- mean((-R*log(P))*n)
    }   
    p <- runif(G)
    p <- optim(p,KL,method="L-BFGS-B")$par
    p <- p^2/sum(p^2)
    yhat[j] <- which.max(p)
  }
  
  class <- res.prob$INFO$level.y[yhat] # クラス予測
  if(is.null(y)){
    res <- list(class=class)
  }
  else{
    res <- list(class=class,error=mean(y!=class,na.rm=TRUE))
  }
  return(res)
}
