predict.decode <- function(res.decode,res.bin,coder,y=NULL){
  ## 関数decodeで学習した復号器を用いてres.binからラベル予測を行う関数
  ## 入力 (res.decode: 関数decode()の結果,
  ##      res.bin: learn.binary() あるいは predict.binary()の結果,
  ##      coder: 符号表, y: 多値のクラスラベル)
  ## 出力 (class: 予測されたクラスラベル, error: 誤り率)
  class <- ncol(coder)
  p <- nrow(coder)
  log.P <- matrix(0,nrow(res.bin$F),class)
  P.0 <- res.bin$P.0
  Z <- c(1,0,-1)
  B <- res.decode$B
  
  for(g in 1:class){ # クラス事後確率の対数を計算
    if(res.decode$model==1){ # モデル1
      B0 <- B[cbind(1:p,match(coder[,g],Z))]
      log.P[,g] <- sign(res.bin$F) %*% B0 -sum(log(exp(B0)+exp(-B0)))+
        log(P.0[g])
    }
    if(res.decode$model==2){ # モデル2
      log.P[,g] <- sign(res.bin$F) %*% B[,g]-sum(log(exp(B[,g])+exp(-B[,g])) )+
        log(P.0[g])
    }
  }
  class <- res.bin$INFO$level.y[max.col(log.P)] # クラス予測
  
  if(is.null(y)){
    res <- list(class=class)
  }
  else{
    res <- list(class=class,error=mean(y!=class,na.rm=TRUE))
  }
  return(res)  
  
}
