predict.decode2 <- function(res,res.bin,coder,y=NULL){
  ## 関数decode2で学習した復号器を用いてres.binからラベル予測を行う関数
  ## 入力 (res: 関数decode2()の結果,
  ##      res.bin: learn.binary(),またはpredict.binary()の結果,
  ##      coder: 符号表, y: 多値のクラスラベル)
  ## 出力 (class: 予測ラベル, class.m: 復号に平均場近似を用いた場合の予測ラベル,
  ##      error: classの誤り率, error.m: class.mの誤り率)

  z.tilde <- sign(res.bin$F)
  class <- ncol(coder)
  p <- ncol(z.tilde)
  n <- nrow(z.tilde)
  
  P <- matrix(0,n,class)
  P.m <- matrix(0,n,class)
  P.0 <- res$P.0
  
  Z.arg.max <- res$Z.arg.max
  Z <- res$Z
  C <- res$C
  logZ.m <- res$logZ.m
  
  for(g in 1:class){
    A <- res$par[[g]]$A
    B <- res$par[[g]]$B
    # 事後確率の計算
    P[,g] <- P.0[g]*exp(calc.exp.arg(z.tilde,A,B)-Z.arg.max[g])/Z[[g]]
    # 平均場近似を用いて復号した場合の事後確率
    P.m[,g] <- P.0[g]*exp(z.tilde %*% as.matrix(C[[g]])-logZ.m[[g]])
  }
  class <- res.bin$INFO$level.y[max.col(P)] # クラス予測
  class.m <- res.bin$INFO$level.y[max.col(P.m)] # クラス予測
  
  if(is.null(y)){ # yが入力されなければ予測値だけを返す
    res <- list(class=class,class.m=class.m)
  }
  else{ # yが入力された時は予測値と誤り率を返す
    error <- mean(class !=y)
    error.m <- mean(class.m !=y)
    res <- list(class=class,class.m=class.m,error=error,error.m=error.m)
  }
  return(res)
}
