predict.loss.decoding <- function(res.bin,coder,mode=1,y=NULL){
  ## ロス関数に基づく復号を行う関数
  ## 入力 (res.bin[リスト]: learn.binary(),あるいはpredict.binary()の結果,
  ##      coder: 符号表, mode:　使用するロス関数の指定, y: 多値のクラスラベル)
  ## 出力 (class: 予測されたクラスラベル, error: 誤り率)

  L <- function(z){ #ロス関数の定義
    if(mode==1){ # 指数ロス
      res <- exp(-z)
    }
    if(mode==2){ # ロジスティックロス
      res <- 1/(1+exp(z))
    }
    res
  }
  
  n <- nrow(res.bin$F)
  class <- ncol(coder)
  p <- nrow(coder)
  F <- matrix(0,n,class)

  for(g in 1:class){ # 各クラスに対するロスの計算
    F[,g] <- apply(L(res.bin$F*matrix(coder[,g],n,p,byrow=TRUE)),1,sum)
  }

  # クラスラベルの予測
  class <- res.bin$INFO$level.y[max.col(-F)]
  
  if(is.null(y)){ # ラベルが入力されなければ予測だけを返す
    res <- list(class=class)
  }
  else{ # ラベルが入力された時は予測と誤り率を返す
    res <- list(class=class,error=mean(y!=class,na.rm=TRUE))
  }
  return(res)
}
