trans.data <- function(x,y,INFO,na.action="all"){
  ## データの加工を行う関数
  ## 入力 (x: 入力, y: ラベル, INFO: 関数make.info()の出力,
  ##      na.action["all" or "remove"]: 欠測値の扱い)
  ## 出力 (x: 加工した入力, y: 加工したラベル, na.index: 欠測があったインデックス)
  
  xnew <- NULL
  for(i in 1:ncol(x)){
    if(is.vector(x[,i])){ # 入力の各列がベクトルであればそのまま追加
      xnew <- cbind(xnew,as.numeric(x[,i]))
    }
    if(is.factor(x[,i])){ # 入力の因子であれば数値化して追加
      xnew <- cbind(xnew,
               outer(x[,i],INFO$level.x[[i]][-1],function(x,y){sign(x==y)}))
    }
  }
  if(is.null(y)){ynew <- NULL} # ラベルが与えられない時は何もしない
  else{ # ラベルが与えられたとき
    ynew <- rep(0,length(y))
    if(length(INFO$level.y)==2){ # 2値の場合は+1,-1へ変換
      ynew <- match(y,INFO$level.y)*2-3
    }
    if(length(INFO$level.y)>2){ # 多値の場合は1,...,Gへ変換
      ynew <- match(y,INFO$level.y)
    }
  }

  if(na.action=="all"){ # 欠測処理をしない場合
    return(list(x=xnew,y=ynew))
  }
  else if(na.action=="remove"){ # 欠測している例題を取り除く
    na.index <- which(apply(is.na(xnew),1,any))
    if(length(na.index)>0){
      xnew <- xnew[-na.index,,drop=FALSE]
      ynew <- ynew[-na.index]
    }
    return(list(x=xnew,y=ynew,na.index=na.index))
  }
}
