library(ada) # adaパッケージの読み込み
library(mlbench) # mlbenchパッケージの読み込み
data(Vowel) # データVowelの読み込み
set.seed(1)
# 990のデータを500個の訓練データ，100個の検証用データ，390個のテストデータに分割する
index1 <- sample(nrow(Vowel),600) # 全データから600個のデータを選択
index2 <- sample(600,100) # 600個から更に100個を選択
val.index <- index1[index2] # 検証用データのインデックス
tra.index <- index1[-index2] # 訓練用データのインデックス
# クラスラベルを2値化                                         
Vowel[,11] <- factor(match(Vowel[,11],levels(Vowel[,11]))>=7,label="class")
# アダブーストの学習, ステップ数は500
res.ada <- ada(Vowel[tra.index,-11],Vowel[tra.index,11],
               Vowel[val.index,-11],Vowel[val.index,11],
               nu=0.5,bag.frac=0.5,iter=500)
# ロジットブーストの訓練
res.log <- ada(Vowel[tra.index,-11],Vowel[tra.index,11],
               Vowel[val.index,-11],Vowel[val.index,11],
               loss="logistic",nu=0.5,bag.frac=0.5,iter=500)

# アダブーストの500ステップ目の判別器の訓練誤差
mean(res.ada$fit != Vowel[tra.index,11]) 
# アダブーストの500ステップ目の判別器の検証誤差
mean(predict(res.ada,Vowel[val.index,]) != Vowel[val.index,11])
# アダブーストの500ステップ目の判別器の予測誤差
mean(predict(res.ada,Vowel[-index1,]) != Vowel[-index1,11])
# アダブーストの検証誤差が最小になるステップでの予測誤差
mean(predict(res.ada,Vowel[-index1,],n.iter=which.min(res.ada$model$err[,3]))
     !=Vowel[-index1,11])

# ロジットブーストの500ステップ目の判別器の訓練誤差
mean(res.log$fit != Vowel[tra.index,11]) 
# ロジットブーストの500ステップ目の判別器の検証誤差
mean(predict(res.log,Vowel[val.index,]) != Vowel[val.index,11])
# ロジットブーストの500ステップ目の判別器の予測誤差
mean(predict(res.log,Vowel[-index1,]) != Vowel[-index1,11])
# ロジットブーストの検証誤差が最小になるステップでの予測誤差
mean(predict(res.log,Vowel[-index1,],n.iter=which.min(res.log$model$err[,3]))
     !=Vowel[-index1,11])

# アダブーストの訓練誤差，検証誤差
plot(log10(1:500),res.ada$model$errs[,1],type="l",xlab="Log of Number of step",
     ylab="Error rate",ylim=range(0.22,0))
lines(log10(1:500),res.ada$model$errs[,3],lty=2,col=2)
title("Error rate of AdaBoost, nu=0.5")
legend(1,0.2,c("Training error","Validation error"),
       col=c(1,2),lwd=rep(1,2),lty=c(1,2),box.col=0)
#dev.copy2eps(file="../Fig/boost-ada.vs.logit.ada.eps",family="Helvetica")
# ロジットブーストの訓練誤差，検証誤差
dev.new()
plot(log10(1:500),res.log$model$errs[,1],type="l",xlab="Log of Number of step",
     ylab="Error rate",ylim=range(0.22,0))
lines(log10(1:500),res.log$model$errs[,3],lty=2,col=2)
title("Error rate of LogitBoost, nu=0.5")
legend(1,0.2,c("Training error","Validation error"),
       col=c(1,2),lwd=rep(1,2),lty=c(1,2),box.col=0)
#dev.copy2eps(file="../Fig/boost-ada.vs.logit.logit.eps",family="Helvetica")
