library(ada) # パッケージadaの読み込み
library(mlbench) # パッケージmlbenchの読み込み
data(Vowel) # データセットVowelの読み込み
set.seed(1)
# 990のデータを500個の訓練データ，100個の検証用データ，390個のテストデータに分割する
index1 <- sample(nrow(Vowel),600) # 全データから600個のデータを選択
index2 <- sample(600,100) # 600個から更に100個を選択
val.index <- index1[index2] # 検証用データのインデックス
tra.index <- index1[-index2] # 訓練用データのインデックス
# クラスラベルを2値化                                         
Vowel[,11] <- factor(match(Vowel[,11],levels(Vowel[,11]))>=7,label="class")
# アダブーストの学習, ステップ数は500
(res <- ada(Vowel[tra.index,-11],Vowel[tra.index,11],
            Vowel[val.index,-11],Vowel[val.index,11],
            nu=1,bag.frac=0.5,iter=500))
which.min(res$model$err[,1]) # 訓練誤差が最小となるステップ
which.min(res$model$err[,3]) # 検証誤差が最小となるステップ

# 500ステップ目の判別器の訓練誤差
mean(res$fit != Vowel[tra.index,11]) 
# 500ステップ目の判別器の検証誤差
mean(predict(res,Vowel[val.index,-11]) != Vowel[val.index,11])

# 500ステップ目での予測誤差
mean(predict(res,Vowel[-index1,],n.iter=500)
     !=Vowel[-index1,11])

# 訓練誤差が最小になるステップでの予測誤差
mean(predict(res,Vowel[-index1,],n.iter=which.min(res$model$err[,1]))
     !=Vowel[-index1,11])

# 検証誤差が最小になるステップでの予測誤差
mean(predict(res,Vowel[-index1,],n.iter=which.min(res$model$err[,3]))
     !=Vowel[-index1,11])

# nu=1,0.1の場合の訓練，検証誤差のプロット
res2 <- ada(Vowel[tra.index,-11],Vowel[tra.index,11],
           Vowel[val.index,-11],Vowel[val.index,11],
           nu=0.1,bag.frac=0.5,iter=500)

# nu=1の場合の訓練誤差，検証誤差のプロット.　横軸はステップ数の対数(底は10)
plot(log10(1:500),res$model$errs[,1],type="l",xlab="Log of Number of step",
     ylab="Error rate",ylim=range(0,0.22))
lines(log10(1:500),res$model$errs[,3],lty=2,col=2)
title("Error rate of AdaBoost, nu=1")
legend(1,0.2,c("Training error","Validation error"),
       col=c(1,2),lwd=rep(1,2),lty=c(1,2),box.col=0)
#dev.copy2eps(file="../Fig/boost-ada.error.eps",family="Helvetica")
# nu=0.1の場合の訓練誤差，検証誤差のプロット. 横軸はステップ数の対数(底は10)
dev.new()
plot(log10(1:500),res2$model$errs[,1],type="l",
     xlab="Log of Number of step",ylab="Error rate",ylim=range(0,0.22))
lines(log10(1:500),res2$model$errs[,3],lty=2,col=2)
title("Error rate of AdaBoost, nu=0.1")
legend(1,0.2,c("Training error","Validation error"),
       col=c(1,2),lwd=rep(1,2),lty=c(1,2),box.col=0)
#dev.copy2eps(file="../Fig/boost-ada.error2.eps",family="Helvetica")
