adaboost.m2 <- function(x,y,T=100,wl="stump.m",na.action="remove",
                        previous.res=NULL,Y.num=1,r.wl=0.5,r.b=0.1){
  ## アダブースト.M2でGクラス問題に対して判別器を構成する関数
  ## 入力 (x: 入力,y: ラベル,T: ステップ数,wl: 弱学習器, スタンプを使用,
  ##      na.action: xにNAが含まれている場合の対応("remove","all"),
  ##      previous.res[adaboost.m2の結果]: 指定した学習結果から再学習,
  ##      Y.num[ベクトル]: スタンプにおいて, 全G個のクラスのうち，Y.num個のクラスと(G-Y.num)個のクラスの2値判別を考える,
  ##      r.wl[0<r.wl<=1の実数]: Y.numで決まる2値判別問題のうち用いる割合,
  ##      r.b[0<r.wl<=1の実数]: スタンプにおいて閾値決定に用いるサンプルの割合)
  ## 出力 (class: 予測されたクラスラベル, error: 誤り率, loss: ロス関数の値,
  ##      F: 判別関数の値, alpha: 弱学習器の係数, WL: 弱学習器, INFO: x,yの情報)
  
  INFO <- make.info(x,y)
  dat <- trans.data(x,y,INFO,na.action)
  INFO$na.index <- dat$na.index
  INFO$na.action <- na.action

  res <- learn.adaboost.m2(dat$x,dat$y,T,wl,Y.num=Y.num,
                           previous.res,r.wl=r.wl,r.b=r.b)
  if(na.action=="remove"){
    if(length(INFO$na.index)>0){
      F.all <- matrix(0,length(y),length(INFO$level.y))
      dat <- trans.data(x[,,drop=FALSE],y,INFO,"all")
      F.all <- predict(res,dat$x,dat$y,T)$F
      res$F <- F.all
    }
  }
  res$INFO <- INFO
  res$class <- INFO$level.y[max.col(res$F)]
  class(res) <- 'adaboost.m2'
  return(res)
}
