library(adabag) # adabagの読み込み
set.seed(1)
# 人工データを生成する関数
gen.data <- function(N){
  dat <- list()
  dat$x <- matrix(runif(N*2)*2-1,N,2)
  dat$y <- sign(dat$x[,1]^2+dat$x[,2]^2-sqrt(2/pi))
  rev.index <- runif(N)<0.1 # ラベル反転するインデックスを決定
  dat$y[rev.index] <- -dat$y[rev.index] # ラベル反転
  dat$y <- factor(dat$y)
  dat
}
tra <- gen.data(200) # 訓練データ
gen <- gen.data(1000) # テストデータ
B <- c(1,10,25,50) # バギング回数の候補
T <- 30 # 30回実験してバギングの平均的な振る舞いを見る
# 学習器である決定木の木の深さの最大値を30に設定
res <- list()
error.tra <- matrix(0,T,length(B))
error.gen <- matrix(0,T,length(B))
for(t in 1:T){
  for(i in 1:length(B)){
    res[[i]] <- bagging(y~.,data.frame(x=tra$x,y=tra$y),mfinal=B[i],maxdepth=30)
    error.tra[t,i] <- mean(res[[i]]$class != tra$y)
    error.gen[t,i] <- mean(predict(res[[i]],data.frame(x=gen$x,y=gen$y))$class
                           != gen$y)
  }
}
# 学習器である決定木の木の深さの最大値を1に設定
res2 <- list()
error2.tra <- matrix(0,T,length(B))
error2.gen <- matrix(0,T,length(B))
for(t in 1:T){
  for(i in 1:length(B)){
    res2[[i]] <- bagging(y~.,data.frame(x=tra$x,y=tra$y),mfinal=B[i],maxdepth=1)
    error2.tra[t,i] <- mean(res2[[i]]$class != tra$y)
    error2.gen[t,i] <- mean(predict(res2[[i]],data.frame(x=gen$x,y=gen$y))$class
                            != gen$y)
  }
}
