source("boost-config.r") # 各関数の読み込み
library(mlbench) # パッケージmlbenchの読み込み
data(BreastCancer) # BreastCancerデータセットの読み込み
dat <- BreastCancer[,-1] # インデックス情報の除去
set.seed(1)
tra.index <- sample(nrow(dat),400) # 400個を訓練データとして用いる
T <- 300 # ブースティングのステップ数
res.tra <- list() # 訓練の結果保存用リスト
res.gen <- list() # テストの結果保存用リスト
# アダブースト
# 10ステップまで学習
res.tra[[1]] <- uboost(dat[tra.index,1:9],dat[tra.index,10],10,Uloss="exp",na.action="remove")
# 11ステップ目以降の続きの学習
res.tra[[1]] <- uboost(dat[tra.index,1:9],dat[tra.index,10],T,Uloss="exp",previous.res=res.tra[[1]],na.action="remove")
# テストデータの予測
res.gen[[1]] <- predict(res.tra[[1]],dat[-tra.index,1:9],dat[-tra.index,10])

# マダブースト
res.tra[[2]] <- uboost(dat[tra.index,1:9],dat[tra.index,10],T,Uloss="mada",na.action="remove")
res.gen[[2]] <- predict(res.tra[[2]],dat[-tra.index,1:9],dat[-tra.index,10])

# イータブースト
res.tra[[3]] <- uboost(dat[tra.index,1:9],dat[tra.index,10],T,Uloss="eta",na.action="remove",eta=0.5)
res.gen[[3]] <- predict(res.tra[[3]],dat[-tra.index,1:9],dat[-tra.index,10])

plot(res.tra[[1]]$error,xlab="Step",ylab="Error",type="l",
     ylim=range(sapply(res.tra,function(z){z$error})) )
for(i in 2:3){
  lines(res.tra[[i]]$error,col=i,lty=i)
}
legend(20,max(res.tra[[1]]$error),c("AdaBoost","MadaBoost","Eta-Boost"),lty=c(1:3),col=c(1:3))
title("Training error")

dev.new()

plot(res.gen[[1]]$error,xlab="Step",ylab="Error",type="l",
     ylim=range(sapply(res.gen,function(z){z$error})))
for(i in 2:3){
  lines(res.gen[[i]]$error,col=i,lty=i)
}
legend(20,max(res.gen[[1]]$error),c("AdaBoost","MadaBoost","Eta-Boost"),lty=c(1:3),col=c(1:3))
title("Test error")

# ラベルにミスラベルがある場合
mis.index <- tra.index[runif(400)<0.1] # 訓練データの10%をラベル反転
dat[mis.index,10] <- levels(dat[,10])[3-c(dat[mis.index,10])]

res2.tra <- list() # 訓練の結果保存用リスト
res2.gen <- list() # テストの結果保存用リスト
# アダブースト
res2.tra[[1]] <- uboost(dat[tra.index,1:9],dat[tra.index,10],T,Uloss="exp",na.action="remove")
# テストデータの予測
res2.gen[[1]] <- predict(res2.tra[[1]],dat[-tra.index,1:9],dat[-tra.index,10])

# マダブースト
res2.tra[[2]] <- uboost(dat[tra.index,1:9],dat[tra.index,10],T,Uloss="mada",na.action="remove")
res2.gen[[2]] <- predict(res2.tra[[2]],dat[-tra.index,1:9],dat[-tra.index,10])

# イータブースト
res2.tra[[3]] <- uboost(dat[tra.index,1:9],dat[tra.index,10],T,Uloss="eta",na.action="remove",eta=0.5)
res2.gen[[3]] <- predict(res2.tra[[3]],dat[-tra.index,1:9],dat[-tra.index,10])

dev.new()
plot(res2.tra[[1]]$error,xlab="Step",ylab="Error",type="l",
     ylim=range(sapply(res2.tra,function(z){z$error})))
for(i in 2:3){
  lines(res2.tra[[i]]$error,col=i,lty=i)
}
legend(20,max(res2.tra[[1]]$error),c("AdaBoost","MadaBoost","Eta-Boost"),lty=c(1:3),col=c(1:3))
title("Training error")

dev.new()

plot(res2.gen[[1]]$error,xlab="Step",ylab="Error",type="l",
     ylim=range(sapply(res2.gen,function(z){z$error})))
for(i in 2:3){
  lines(res2.gen[[i]]$error,col=i,lty=i)
}
legend(20,max(res2.gen[[1]]$error),c("AdaBoost","MadaBoost","Eta-Boost"),lty=c(1:3),col=c(1:3))
title("Test error")
