learn.uboost <- function(x,y,T,wl="stump",Uloss,previous.res,eta){
  if(Uloss=="exp"){
    U <- function(z){
      exp(z)
    }
    Udash <- function(z){
      exp(z)
    }
  }
  if(Uloss=="logit"){
    U <- function(z){
      log(1+exp(z))
    }
    Udash <- function(z){
      exp(z)/(1+exp(z))
    }
  }
  if(Uloss=="eta"){
    U <- function(z){
      (1-eta)*exp(z)+eta*z
    }
    Udash <- function(z){
      (1-eta)*exp(z)+eta
    }
  }
  if(Uloss=="mada"){
    U <- function(z){
      sign(z>0)*(z+0.5)+sign(z<=0)*exp(2*z)/2
    }
    Udash <- function(z){
      sign(z>0)+sign(z<=0)*exp(2*z)
    }
  }
  
  t0 <- 1
  
  p <- ncol(x)
  n <- nrow(x)

  e <- rep(0,T)
  error <- rep(0,T)
  loss <- rep(0,T)
  
  alpha <- rep(0,T)
  WL <- list()

  w <- rep(1/n,n)
  F <- rep(0,n)

  if(!is.null(previous.res)){
    T.prev <- length(previous.res$alpha)
    if(T.prev >=T){
      return(previous.res)
    }
    else if(T.prev >length(previous.res$WL)){
      cat("Weighted error of weak learner is 0 in given previous result\n")
      return(previous.res)
    }
    else{
      t0 <- T.prev+1

      error <- previous.res$error
      loss <- previous.res$loss
      
      WL <- previous.res$WL
      alpha[1:T.prev] <- previous.res$alpha
      for(t in 1:T.prev){
        F <- F+alpha[t]*predict(WL[[t]],x)
      }
      w <- Udash(-F*y)
      w <- w/sum(w)  #Normalization of weight
    }
  }

  cat(c(0,"|__________|",100,"\n",sep=""))
  cat(" |")

  for(t in t0:T){
    
    WL[[t]] <- list()
    class(WL[[t]]) <- wl

    ## weak learner
    WL[[t]] <- learn.stump(WL[[t]],x,y,w)
    fval <- predict(WL[[t]],x)
    
    ## Calculation of alpha
    LOSS <- function(z){
      mean(U(-(F+z*fval)*y))
    }

    alpha[t] <- optim(0,LOSS,method="L-BFGS-B")$par
    
    ## Update of discriminant function
    F <- F+alpha[t]*fval
    
    ##  Update of weight
    w <- Udash(-F*y)
    w <- w/sum(w)  #Normalization of weight
    
    error[t] <-  mean(sign(y != sign(F) ))
    loss[t] <- mean( U(-F*y))

    if( ((t-t0+1)%% ((T-t0+1)/10))<1){
      cat("*")
    }     
    
   }

  cat(c("|\n"))
  res <-  list(error=error,loss=loss,F=F,alpha=alpha,WL=WL)
  class(res) <- 'uboost'
  return(res)
}
