lfda <- function(x,y,mode="exp",dimen=NULL,lambda=1,k=5){
  ## 入力x, ラベルyを受け取り，局所フィッシャー判別分析を行う
  ## 入力 (x: 入力, y: 教師ラベル, mode: 類似度の定義 exp or nn, dimen: 判別する際に用いる次元,
  ##      lambda: mode="exp"の場合の類似度調整パラメーター, k: mode="nn"の時のパラメーター
  ## 出力 (INFO: 入力，ラベルの情報, Fx[リスト]: 学習した２値判別器, F[行列]: 入力xに対するFxの出力, method: 2値判別器として用いた方法, P.0: クラスラベルの事前確率)

  INFO <- make.info(x,y)
  dat <- trans.data(x,y,INFO,"remove")
  INFO$na.index <- dat$na.index
  INFO$na.action <- na.action

  if(is.null(dimen)){
    dimen <- ncol(dat$x)
  }
  n <- nrow(dat$x)

  ny <- c(table(dat$y))
  CLASS <- length(INFO$level.y)
  y0 <- match(dat$y,unique(dat$y))
  A <- matrix(0,n,n)
  library(MASS)
  
  if(mode=="exp"){
    D <- outer(apply(dat$x^2,1,sum),apply(dat$x^2,1,sum),function(x,y){x+y})-2*( dat$x%*% t(dat$x))
    A <- exp(-lambda*D)
  }
  if(mode=="nn"){
    A <- outer(apply(dat$x^2,1,sum),apply(dat$x^2,1,sum),function(x,y){x+y})-2*( dat$x%*% t(dat$x))
    D1 <- t(apply(A,1,function(z){order(z)[1:k]}))
    D1 <- c((D1-1)*n+c(1:n))
    A[D1] <- (1-(A[D1]/A[D1[((k-1)*n+1):(k*n)]])^(1.5))^3
    #A[D1] <- 1-A[D1]/A[D1[((k-1)*n+1):(k*n)]]
    A[-D1] <- 0
  }
  Ww <- A*outer(y0,y0,function(x,y){x==y})/ny[y0]
  Wb <- outer(y0,y0,function(x,y){x==y})*(1/n-1/ny[y0])+
    outer(y0,y0,function(x,y){x!=y})/n
  Wb <- Wb*A
  
  Vw <- t(dat$x) %*% (diag(apply(Ww,1,sum)+apply(Ww,2,sum))-Ww-t(Ww)) %*% dat$x
  Vb <- t(dat$x) %*% (diag(apply(Wb,1,sum)+apply(Wb,2,sum))-Wb-t(Wb)) %*% dat$x

  W <- eigen(ginv(Vw) %*% Vb)
  #W <- eigen(solve(Vw) %*% Vb)
  X <- dat$x %*% W$vectors[,1:dimen]
  res <- list(w=W$vectors,lambda=W$values,X=X)
  class(res) <- 'lfda'
  return(res)
}

