library(mlbench) # for Vowel
data(Vowel)
knndist <- function(x,dat,k){
  # xから第k近傍までの距離を計算する関数
  # 入力 (x[行列]: 入力点, dat[行列]:データ点, k:近傍数)
  # 出力 ([ベクトル]: 距離)
  m<-nrow(x)
  n<-nrow(dat)
  tmp<-as.matrix(dist(rbind(as.matrix(dat),as.matrix(x))))
  return(apply(tmp[(1:m)+n,1:n],1,function(z){return(sort(z)[k])}))
}
## fieldsパッケージのrdistを用いる場合の関数の定義
##library(fields) # for rdist
##knndist <- function(x,dat,k){
##  return(apply(rdist(x,dat),1,function(z){return(sort(z)[k])}))
##}
#
knndens <- function(x,dat,k){
  # 点xでの最近傍法による密度を計算する関数
  # 入力 (x[行列]: 入力点, dat[行列]:データ点, k:近傍数)
  # 出力 ([ベクトル]: 密度)
  d<-ncol(dat) # 次元数
  n<-nrow(dat) # データ数
  V<-2*pi^(d/2)/gamma(d/2)/d # 単位球の体積
  return(k/n/(V*knndist(x,dat,k)))
}
# 密度の表示 
plot(density(Vowel$V8),ylim=c(0,0.9),
	main="Nearest Neighbor Density Estimation",lwd=2)
rug(Vowel$V8)
# 定義域を分割して計算
x<-seq(min(Vowel$V8)-0.2,max(Vowel$V8)+0.2,length=1000)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),20),col=2,lty=2,lwd=2)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),40),col=3,lty=3,lwd=2)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),80),col=4,lty=4,lwd=2)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),160),col=5,lty=5,lwd=2)
legend(1.5,0.9,legend=c("kernel","k=20","k=40","k=80","k=160"),
	col=1:5,lty=1:5,lwd=2)
# 拡大図
dev.new()
plot(density(Vowel$V8),xlim=c(0,1),ylim=c(0.3,0.9),
	main="Enlargement",lwd=2)
x<-seq(0,1,length=100)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),20),col=2,lty=2,lwd=2)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),40),col=3,lty=3,lwd=2)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),80),col=4,lty=4,lwd=2)
lines(x,knndens(as.matrix(x),as.matrix(Vowel$V8),160),col=5,lty=5,lwd=2)
legend(0,0.9,legend=c("kernel","k=20","k=40","k=80","k=160"),
	col=1:5,lty=1:5,lwd=2)
