calc.auc <- function(F,y,r=NULL,graphic=FALSE){
  # 判別に用いる値Fとラベルyを受け取ってAUC(rが指定された場合はpAUC)を計算する関数
  # 入力 (F[ベクトル]: 入力に対する判別関数の値（符号をとる前の値)，y: ラベル,
  #      r[ベクトル]:　指定された場合はFPRがr以下のpAUCを計算,
  #      graphic: TRUEの場合FPR,TPRを計算)
  # 出力 (auc: AUC値, FP: FPR値, TP:　TPR値，pauc: pAUC値)
  y <- match(y,levels(factor(y)))*2-3
  auc <- mean(outer(F[y==1],F[y==-1],function(x,y){x>y}))
  
  if(graphic==TRUE | !is.null(r)){
    F.sorted <- sort(unique(F),decreasing=TRUE)
    n <- length(F.sorted)
    C <- c(F.sorted[1]+1,(F.sorted[-1]+F.sorted[-n])/2,F.sorted[n]-1)

    FP <- rep(0,length(C))
    TP <- rep(0,length(C))
    for(i in 1:length(C)){
      FP[i] <- mean(F[y==-1]>C[i])
      TP[i] <- mean(F[y==1]>C[i])
    }
    
    #if(graphic==TRUE){
    #  plot(FP,TP,xlab="FP",ylab="TP",type="o",lwd=1)
    #}
    
    if(!is.null(r)){
      pauc <- rep(0,length(r))
      for(i in 1:length(r)){
        pFP <- c(FP[FP<=r[i]],r[i])
        pTP <- TP[FP<=r[i]]
        pTP <- c(pTP,pTP[length(pTP)])
        pauc[i] <- sum((pFP[-1]-pFP[-length(pFP)])*pTP[-1])
      }
      return(list(auc=auc,pauc=pauc,FP=FP,TP=TP))
    }
    else{
      return(list(auc=auc,FP=FP,TP=TP))
    }
  }
  else{
    return(list(auc=auc))
  }
}
