library(mlbench)
library(kernlab)
n <- 200  # データ数 200
k <- 10   # 交互検証法で 10 個のデータセットに分割
dat <- mlbench.spirals(n, cycles=1.2, sd=0.16)  # データ生成
x <- dat$x; y <- dat$classes
idx <- sample(k,n,replace=TRUE)   # データをランダムにk分割する
sigma.list <- seq(0.1,15,by=0.1)  # sigma の候補： 0.1 - 15 まで 0.1 刻み
cv <- c()
for(sigma in sigma.list){    # sigma.list の値を順番に sigma に代入
   err_estimate <- c()
   for (j in 1:k){            # j を 1 から k まで繰り返す
     tmpx <- x[idx!=j,]; tmpy <- y[idx!=j] # 交互検証法で推定に使うデータ
     rbfsvm<-ksvm(tmpx,tmpy,type="C-svc",kernel="rbfdot",kpar=list(sigma=sigma))
     rbfpred <- predict(rbfsvm,x[idx==j,]) # 除いたデータ上で予測
     err_estimate <- c(err_estimate,mean(y[idx==j]!=rbfpred)) #予測誤差の推定
   }
   cv <- c(cv, mean(err_estimate))
 }
sigma.list[which.min(cv)]  # 交差検証法で選ばれた sigma の値

