source("eval-calc.auc.r") # 関数の読み込み
library(MASS) # MASSパッケージの読み込み
library(mlbench) # mlbenchパッケージの読み込み
set.seed(1)
dat <- mlbench.circle(200) # データの生成
res.lda <- lda(dat$x,dat$classes) # 線形判別器の訓練
res.qda <- qda(dat$x,dat$classes) # 2次判別器の訓練

# 誤り率
mean(predict(res.lda,dat$x)$class != dat$classes) # 線形判別器の訓練誤差
mean(predict(res.qda,dat$x)$class != dat$classes) # 2次判別器の訓練誤差

# 線形判別器のAUC
calc.auc(predict(res.lda,dat$x)$posterior[,2],dat$classes)$auc
# 2次判別器のAUC
calc.auc(predict(res.qda,dat$x)$posterior[,2],dat$classes)$auc 

# 線形判別器のpACU (r=0.1,0.2)
calc.auc(predict(res.lda,dat$x)$posterior[,2],dat$classes,r=c(0.1,0.2))$pauc
# 2次判別器のpAUC  (r=0.1,0.2)
calc.auc(predict(res.qda,dat$x)$posterior[,2],dat$classes,r=c(0.1,0.2))$pauc 

dat.gen <- mlbench.circle(2000) # テストデータの生成
#　テストデータに対する性能を検証
# 線形判別器のAUC
calc.auc(predict(res.lda,dat.gen$x)$posterior[,2],dat.gen$classes)$auc
# 2次判別器のAUC
calc.auc(predict(res.qda,dat.gen$x)$posterior[,2],dat.gen$classes)$auc 

# ROCカーブを書くためにFPR, TPRを計算
res.lda.auc <- calc.auc(predict(res.lda,dat$x)$posterior[,2],
                        dat$classes,graphic=TRUE)
res.qda.auc <- calc.auc(predict(res.qda,dat$x)$posterior[,2],
                        dat$classes,graphic=TRUE)
res2.lda.auc <- calc.auc(predict(res.lda,dat.gen$x)$posterior[,2],
                         dat.gen$classes,graphic=TRUE)
res2.qda.auc <- calc.auc(predict(res.qda,dat.gen$x)$posterior[,2],
                         dat.gen$classes,graphic=TRUE)
# データセット
plot(dat$x,col=dat$classes,xlab="",ylab="",pch=c(21:22)[c(dat$classes)])
dev.new()
plot(res.lda.auc$FP,res.lda.auc$TP,xlab="FP",ylab="TP",type="l",lwd=2)
lines(res.qda.auc$FP,res.qda.auc$TP,col=2,lty=2,lwd=2)
lines(res2.lda.auc$FP,res2.lda.auc$TP,col=3,lty=3,lwd=2)
lines(res2.qda.auc$FP,res2.qda.auc$TP,col=4,lty=4,lwd=2)
legend(0.4,0.3,c("Training ROC curve of lda","Trainng ROC curve of qda",
                 "Test ROC curve of lda","Test ROC curve of qda"),
       lwd=rep(2,4),col=c(1:4),lty=c(1:4),box.col=0)
