source("eval-calc.fmeasure.r") # 関数の読み込み
library(klaR) # klaRパッケージの読み込み
library(mlbench) # mlbenchパッケージの読み込み
set.seed(1)
dat <- mlbench.circle(200) # 訓練データの生成
M <- 20 
fmeasure <- rep(0,M)
pre <- rep(0,M)
rec <- rep(0,M)
err <- rep(0,M)
param <- seq(0,0.9,length=M) # 候補のパラメーター集合
for(i in 1:M){
  # 正則化判別分析
  res <- rda(dat$x,dat$classes,lambda=param[i],gamma=0.5)
  # 訓練誤差
  err[i] <- mean(predict(res,dat$x)$class != dat$classes)
  # F値の計算
  res2 <- calc.fmeasure(predict(res,dat$x)$posterior[,2]-0.5,dat$classes)
  fmeasure[i] <- res2$fmeasure #F値
  pre[i] <- res2$pre # 適合率
  rec[i] <- res2$rec # 再現率
}
plot(param,err,xlab="lambda",ylab="Error",type="o")
dev.new()
plot(param,fmeasure,xlab="lambda",ylab="F-value",type="o",
     ylim=range(c(fmeasure,pre,rec)),pch=21)
lines(param,pre,type="o",lty=2,col=2,pch=22)
lines(param,rec,type="o",lty=3,col=3,pch=23,lwd=3)
legend(0,0.7,c("F-value","Precision","Recall"),lty=rep(1:3),col=c(1:3),
       lwd=c(1,1,3),pch=c(21:23),box.col=0)
