library(fields)  # rdist
library(MASS)    # ginv
aknn <- function(x, xtest, y, k=1, km=min(50,nrow(x)), epsilon=1){
  # 適応的k-近傍法で判別を行う関数
  # 入力(x:入力データ, y:出力データ，xtest:テスト点，
  #      k: k-近傍法, km: V_B, V_W の推定に使うデータ数,
  #      epsilon: 補正項のパラメータ)
  # 出力(predcl: テスト点の予測ラベル)
  dist_mat <- rdist(x, xtest)    
  predcl <- c()
  for (j in 1:nrow(xtest)){
    subidx <- order(dist_mat[,j])[1:km]
    subtrain <- x[subidx,]; subcl <- y[subidx]
    ## クラス間分散とクラス内分散の計算
    adapt_mat <- adaptive_cov_matrix(subtrain,subcl,epsilon=epsilon)
    ## カルバック・ライブラー距離による近傍点の探索
    dx <- t(x)-xtest[j,]
    idx <- order(colSums((adapt_mat %*% dx) * dx))[1:k]
    predcl <- c(predcl,names(which.max(table(y[idx]))))  ## ラベルの予測
  }
  predcl
}
adaptive_cov_matrix <- function(subtrain,subcl,epsilon=1){
  # 局所距離を定める正定値行列を計算する関数．
  # 入力(subtrain:入力データ, subcl:出力ラベル,epsilon: 補正項のパラメータ)
  # 出力:局所距離を定める正定値行列
  p <- ncol(subtrain)
  localcl <- table(subcl)             
  label_prob <- localcl/sum(localcl)  # 局所データのラベル確率
  barx <- colMeans(subtrain)          # 局所データの平均
  W <- matrix(0,p,p); B <- matrix(0,p,p)
  # V_W と V_B の計算
  for (label in levels(subcl)){
    matchlabel <- subcl==label
    if (any(matchlabel)){
      subx <- matrix(subtrain[matchlabel,],sum(matchlabel),p)
      if (nrow(subx)>1)
        W <- W + label_prob[label]*cov(subx)
      bias <- colMeans(subx)-barx
      B <- B + label_prob[label]*(bias %o% bias)
    }
  }
  invW <- ginv(W)
  invW %*% B %*% invW + epsilon*invW
}
