library(class)      # knn, knn.cv
library(mlbench)    # mlbench.corners
knn_est_k <- function(x,cl,klist){
  # k-近傍法に関する交差検証法を行う関数
  # 入力(x[行列]: 入力データ，cl:出力ラベル，
  #      klist[ベクトル]: 学習に用いる k の集合)
  # 出力(cv: 各 k に対する予測誤差の推定値,
  #       k: 交差検証法で選ばれた最適な k)
  cv <- c()
  for (k in klist){
    kn <- knn.cv(x,cl,k=k)
    cv <- c(cv, mean(kn!=cl))   # k-近似法の予測誤差の推定値
  }
  list(cv=cv, k=which.min(cv))  # Error(k) と最小を達成する k を出力
}

dim <- 2; n <- 300; sd <- 0.25  # データの分布の設定
p <- mlbench.corners(n,d=dim,sd=sd) # 学習データ
x <- p$x; y <- p$classes
klist <- seq(1,190,by=5)
kcv <- knn_est_k(x,y,klist)  # 最適なkの推定
ptest <- mlbench.corners(10000,d=dim,sd=sd) # テストデータ
xtest <- ptest$x; ytest <- ptest$classes
# 予測誤差の計算
test_err <- c()
for (k in klist)
  test_err <- c(test_err, mean(knn(x,xtest,y,k=k)!=ytest))
# Error(k) と予測誤差をプロット
errs <- c(kcv$cv,test_err); ylim=c(min(errs),max(errs))
plot(klist,kcv$cv,xlab="k",ylab="Error(k) and prediction error",ylim=ylim,
     main="cross-validation and prediction error",type="l",lwd=3,col=2,lty=1)
lines(klist,test_err,lwd=3,col=4,lty=2)
legend(0,max(errs),c("Error(k)","prediction error"),col=c(2,4),lty=c(1,1),lwd=c(3,3))
