library(class)      # knn, knn.cv
library(mlbench)    # mlbench.corners
test_error_calc <- function(sigma,dim=2){
  # mlbench.corners で生成されたデータに対して
  # ベイズ誤差 (Error) の値を計算する関数．
  # 入力(sigma: 真の分布の標準偏差, dim: 入力の次元)
  # 出力(e: ベイズ誤差の推定値，ub: 1-knnの誤り率の上界)
  G <- 2^dim  # ラベル数
  error <- 1-pnorm(1/(2*sigma))^dim   # ベイズ誤差
  ubound <- 2*error-error^2 *G/(G-1)  # 上界
  list(e=error, ub=ubound)
}
dim <- 3; sigma <- 0.3
nlist <- seq(20,200,by=20)
tp <- mlbench.corners(10000,d=dim,sd=sigma) # テストデータの生成
xtest <- tp$x; ytest <- tp$classes
test_error <- c(); cv_error <- c()
for (n in nlist){
  error <- c(); cv <- c()
  for (i in 1:20){
    ## 学習データの生成
    p <- mlbench.corners(n,d=dim,sd=sigma); x <- p$x; y <- p$classes
    kn <- knn(x,xtest,y,k=1)
    ypred <- match(kn,levels(factor(kn)))
    error <- c(error, mean(ypred != ytest))
    cv <- c(cv, knn_est_k(x,y,1)$cv)
  }
  test_error <- c(test_error, mean(error))
  cv_error <- c(cv_error, mean(cv))
}
# プロット
par(ps=16)
te <- test_error_calc(sigma,dim)
ylim <- c(te$e-0.05,max(cv_error))
plot(nlist,rep(te$e,length(nlist)),lwd=2,lty=3,type="l",
     ylim=ylim,xlab="n",ylab="error")
lines(nlist,rep(te$ub,length(nlist)),lwd=2,lty=2)
lines(nlist,test_error,type="o",col=2,lwd=3,lty=4)
lines(nlist,cv_error,type="o",col=4,lwd=3,lty=5)
legend(130,max(ylim),
       c("Bayes error","upper bound","prediction error","cross validation"),
       col=c(1,1,2,4),lty=c(3,2,4,5),lwd=c(2,2,3,3),y.intersp=1.5)
