data(iris) # データセットirisの読み込み
# ラベルを2値に変換
iris$Species <- factor(c(rep("set+vir",50),rep("ver",50),rep("set+vir",50)),
                       levels=c("set+vir","ver"))
names(iris)[-5] <- abbreviate(names(iris))[-5] # 特徴量の名前を短縮
head(iris) # iris データセット
set.seed(1)
index <- sample(nrow(iris),nrow(iris)*0.7) # 訓練データのインデックスを作成
res <- glm(Species~.,iris[index,],family=binomial) # glmを用いたロジスティック判別
res$coefficients # 各入力変数の係数
logLik(res) # 対数尤度
# ラベルの予測
yhat <- levels(iris$Species)[(predict(res,iris,type="response")>0.5)+1]
mean(yhat[index] !=iris$Species[index]) # 訓練誤差
mean(yhat[-index] != iris$Species[-index]) # 予測誤差
# 判別の内訳, 行は元のラベル，列は予測したラベル
table(true=iris$Species,prediction=yhat)
# クラス事後確率のプロット
plot(predict(res,iris),predict(res,iris,type="response"),pch=c(21,24)[iris[,5]],
     col=iris[,5],xlab=expression(beta^T *x),ylab="Posterior probability")
dev.new()
# フィッティングの残差
plot(res,which=1,col=iris[,5],pch=c(21,24)[c(iris[,5])])
dev.new()
# データの散布図
plot(iris[,-5],col=iris[,5],pch=c(21,24)[c(iris[,5])])
dev.new()
# 予測したラベルの散布図
yhat0 <- match(yhat,levels(iris[,5]))
plot(iris[,-5],col=yhat0,pch=c(21,24)[yhat0])

