data(iris) # データセットirisの読み込み
library(nnet)
# ラベルを2値に変換
names(iris)[-5] <- abbreviate(names(iris))[-5] # 特徴量の名前を短縮
set.seed(1)
index <- sample(nrow(iris),nrow(iris)*0.7) # 訓練データのインデックスを作成
SIZE <- c(1:10)*2 # 隠れ層のサイズの候補
tra.err <- rep(1,length(SIZE))
gen.err <- rep(1,length(SIZE))
# 隠れ層のサイズに対する誤差の挙動を観察する
for(i in 1:length(SIZE)){
  res <- nnet(Species~.,iris[index,],size=SIZE[i])
  yhat <- predict(res,iris,type="class")
  tra.err[i] <- mean(yhat[index] != iris$Species[index]) # 訓練誤差
  gen.err[i] <- mean(yhat[-index] != iris$Species[-index]) # 予測誤差
}
plot(SIZE,tra.err,xlab="Size of hidden unit",ylab="Error",type="o",pch=21,
     ylim=range(tra.err,gen.err))
lines(SIZE,gen.err,type="o",lty=2,col=2,pch=22)
legend(10,max(c(gen.err)),c("Training error","Prediction error"),
       col=c(1,2),lwd=c(1,2),lty=c(1,2),pch=c(21,22),box.col=0)

# 2値判別の場合
iris$Species <- factor(c(rep("set+vir",50),rep("ver",50),rep("set+vir",50)),
                       levels=c("set+vir","ver"))
SIZE <- c(1:10)*3 # 隠れ層のサイズの候補
tra.err <- rep(1,length(SIZE))
gen.err <- rep(1,length(SIZE))
# 隠れ層のサイズに対する誤差の挙動を観察する
for(i in 1:length(SIZE)){
  res <- nnet(Species~.,iris[index,],size=SIZE[i])
  yhat <- predict(res,iris,type="class")
  tra.err[i] <- mean(yhat[index] != iris$Species[index]) # 訓練誤差
  gen.err[i] <- mean(yhat[-index] != iris$Species[-index]) # 予測誤差
}
dev.new()
plot(SIZE,tra.err,xlab="Size of hidden unit",ylab="Error",type="o",pch=21,
     ylim=range(tra.err,gen.err))
lines(SIZE,gen.err,type="o",lty=2,col=2,pch=22)
legend(10,max(tra.err),c("Training error","Prediction error"),
       col=c(1,2),lwd=c(1,2),lty=c(1,2),pch=c(21,22),box.col=0)
