library(rpart) # パッケージrpartの読み込み
library(mlbench) # パッケージmlbenchの読み込み
data(Glass) # Glassデータセットの読み込み
set.seed(1)
# 訓練に用いるデータのインデックスを抽出
tra.index <- sample(nrow(Glass),nrow(Glass)*0.7)
CP <- seq(0,0.95,length=10) # 複雑度パラメーターの候補
tra <- rep(0,length(CP))
gen <- rep(0,length(CP))

# 複雑度パラメーターを変化させながら決定木を学習
for(i in 1:length(CP)){
  res <- rpart(Type~.,Glass[tra.index,],method="class",cp=CP[i])
  yhat <- predict(res,Glass,type="class") #ラベルの予測
  tra[i] <- mean(yhat[tra.index]  !=Glass$Type[tra.index])
  gen[i] <- mean(yhat[-tra.index] !=Glass$Type[-tra.index])
}
plot(CP,tra,xlab="cp",ylab="Error",ylim=range(c(tra,gen)),pch=21)
points(CP,gen,pch=22,col=2)
legend(0.5,0.5,c("Training error","Test error"),
       col=c(1,2),pch=c(21,22),box.col=0)

MD <- c(1:10) # 木の深さの最大値の候補
tra <- rep(0,length(MD))
gen <- rep(0,length(MD))

# 木の深さの最大値を変化させながら決定木を学習
for(i in 1:length(MD)){
  res <- rpart(Type~.,Glass[tra.index,],method="class",maxdepth=MD[i])
  yhat <- predict(res,Glass,type="class") #ラベルの予測
  tra[i] <- mean(yhat[tra.index]  !=Glass$Type[tra.index])
  gen[i] <- mean(yhat[-tra.index] !=Glass$Type[-tra.index])
}
dev.new()
plot(MD,tra,xlab="Max depth",ylab="Error",ylim=range(c(tra,gen)),pch=21)
points(MD,gen,pch=22,col=2)
legend(5,0.55,c("Training error","Test error"),
       col=c(1,2),pch=c(21,22),box.col=0)

