library(rpart) # パッケージrpartの読み込み
library(mlbench) # パッケージmlbenchの読み込み
data(Glass) # Glassデータセットの読み込み
set.seed(1)
# 訓練に用いるデータのインデックスを抽出
tra.index <- sample(nrow(Glass),nrow(Glass)*0.7)
# 決定木を学習
res <- rpart(Type~.,Glass[tra.index,],method="class")
yhat <- predict(res,Glass,type="class") #ラベルの予測

mean(yhat[tra.index]!=Glass$Type[tra.index]) # 訓練誤差
mean(yhat[-tra.index]!=Glass$Type[-tra.index]) # 予測誤差

table(true=Glass$Type,prediction=yhat) # 判別結果の細かい内訳

#ロス行列を定義
LOSS <- matrix(1,length(levels(Glass$Type)),length(levels(Glass$Type)))
LOSS[,c(3:5)] <- 100
diag(LOSS) <- 0
LOSS # 設定したロス行列
# ロス行列を用いて決定木を学習
res2 <- rpart(Type~.,Glass[tra.index,],method="class",parms=list(loss=LOSS))
yhat2 <- predict(res2,Glass,type="class") # ラベルの予測

mean(yhat2[tra.index]!=Glass$Type[tra.index]) # 訓練誤差
mean(yhat2[-tra.index]!=Glass$Type[-tra.index]) # 予測誤差

table(true=Glass$Type,prediction=yhat2) # 判別結果の細かい内訳

# 事前確率を一様に設定して決定木を学習
res3 <- rpart(Type~.,Glass[tra.index,],method="class",
              parms=list(prior=rep(1/6,6)))
yhat3 <- predict(res3,Glass,type="class") # ラベルの予測

mean(yhat3[tra.index]!=Glass$Type[tra.index]) # 訓練誤差
mean(yhat3[-tra.index]!=Glass$Type[-tra.index]) # 予測誤差

table(true=Glass$Type,prediction=yhat3) # 判別結果の細かい内訳
